<?php

namespace App\Controllers;

use App\Models\UserModel;
use App\Models\RequestModel;

class UserManagement extends BaseController
{
    protected $userModel;
    protected $requestModel;

    public function __construct()
    {
        $this->userModel = new UserModel();
        $this->requestModel = new RequestModel();
    }

    public function index()
    {
        if (session()->get('role') != 'admin') {
            return redirect()->to('/dashboard')->with('error', 'Access denied.');
        }

        $data['users'] = $this->userModel->findAll();
        return view('user_management/index', $data);
    }
  
  	public function add()
    {
        if (session()->get('role') != 'admin') {
            return redirect()->to('/dashboard')->with('error', 'Access denied.');
        }

        if ($this->request->getMethod() === 'post') {
            $rules = [
                'username' => 'required|min_length[3]|is_unique[users.username]',
                'email' => 'required|valid_email|is_unique[users.email]',
                'password' => 'required|min_length[6]',
                'role' => 'required|in_list[user,agent,admin]',
            ];

            if ($this->validate($rules)) {
                $this->userModel->save([
                    'username' => $this->request->getPost('username'),
                    'email' => $this->request->getPost('email'),
                    'password' => password_hash($this->request->getPost('password'), PASSWORD_DEFAULT),
                    'role' => $this->request->getPost('role'),
                    'is_active' => 1,
                ]);

                return redirect()->to('/user-management')->with('message', 'User added successfully');
            } else {
                return view('user_management/add', ['validation' => $this->validator]);
            }
        }

        return view('user_management/add');
    }
  
  	public function edit($id)
    {
        if (session()->get('role') != 'admin') {
            return redirect()->to('/dashboard')->with('error', 'Access denied.');
        }

        $user = $this->userModel->find($id);

        if ($this->request->getMethod() === 'post') {
            $rules = [
                'username' => "required|min_length[3]|is_unique[users.username,id,$id]",
                'email' => "required|valid_email|is_unique[users.email,id,$id]",
                'role' => 'required|in_list[user,agent,admin]',
            ];

            if ($this->request->getPost('password')) {
                $rules['password'] = 'min_length[6]';
            }

            if ($this->validate($rules)) {
                $data = [
                    'username' => $this->request->getPost('username'),
                    'email' => $this->request->getPost('email'),
                    'role' => $this->request->getPost('role'),
                ];

                if ($this->request->getPost('password')) {
                    $data['password'] = password_hash($this->request->getPost('password'), PASSWORD_DEFAULT);
                }

                $this->userModel->update($id, $data);

                return redirect()->to('/user-management')->with('message', 'User updated successfully');
            } else {
                return view('user_management/edit', ['validation' => $this->validator, 'user' => $user]);
            }
        }

        return view('user_management/edit', ['user' => $user]);
    }
  
  	public function delete($id)
    {
        if (session()->get('role') != 'admin') {
            return $this->response->setJSON(['success' => false, 'message' => 'Access denied.']);
        }

        if ($this->userModel->delete($id)) {
            return $this->response->setJSON(['success' => true, 'message' => 'User deleted successfully.']);
        } else {
            return $this->response->setJSON(['success' => false, 'message' => 'Failed to delete user.']);
        }
    }


    public function toggleUserStatus($userId)
    {
        if (session()->get('role') != 'admin') {
            return $this->response->setJSON(['success' => false, 'message' => 'Access denied.']);
        }

        $user = $this->userModel->find($userId);
        if (!$user) {
            return $this->response->setJSON(['success' => false, 'message' => 'User not found.']);
        }

        $newStatus = $user['is_active'] ? 0 : 1;
        $this->userModel->update($userId, ['is_active' => $newStatus]);

        return $this->response->setJSON(['success' => true, 'message' => 'User status updated.', 'newStatus' => $newStatus]);
    }

    public function userRequests($userId)
    {
        if (session()->get('role') != 'admin') {
            return redirect()->to('/dashboard')->with('error', 'Access denied.');
        }

        $data['user'] = $this->userModel->find($userId);
        $data['requests'] = $this->requestModel->getUserRequests($userId);
        return view('user_management/user_requests', $data);
    }
}